# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import logging
from six import StringIO
import datetime

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String

logger = logging.getLogger("driverng.agilent_masshunter")


class AgilentMassHunterBase(Splitted):

    SEPARATOR = ","


class AgilentMassHunterResult(object):
    FIELDS = [
        String("name"),
        String("barcode"),
        String("date_file"),
        String("name_method"),
        String("volume"),
        String("instrument"),
        String("type"),
        String("concentration"),
        String("method"),
        String("Date_acquittement"),
        String("Acq_operator"),
        String("position"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            setattr(self, field.name, line[i])


class AgilentMassHunterMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: AgilentMassHunterResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.date_resu = None
        self.operator = None
        for line in reader:
            if not len(line):
                continue
            if "Analysis Time Stamp" in line[0].strip() and len(line) == 2:
                self.date_resu = datetime.datetime.strptime(
                    line[1].strip(), "%m/%d/%Y %I:%M %p"
                )
            if "Analyst Name" in line[0].strip() and len(line) == 2:
                self.operator = line[1].strip()
            if len(line) > 7:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):

    SPEC = {"code": "agilent_masshunter", "label": "Agilent MassHunter CSV"}

    def parse(self, data):
        obj = AgilentMassHunterMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.name.strip() == "Name":
                continue
            if m.type.strip() in ("QC", "CAL"):
                sample = worklist.get_or_create(
                    sample_id=m.barcode, control_lot=m.barcode, type="control"
                )
            else:
                sample = worklist.get_or_create(
                    sample_id=m.barcode, rack=m.position, type="patient"
                )
            if m.concentration:
                sample.results.add(
                    code=m.name_method,
                    value=m.concentration.strip(),
                    date=message.date_resu,
                )
        return worklist
